@@ -23,11 +23,6 @@ from os.path import join as opj, exists
 from os.path import relpath
 from os.path import normpath
 import sys
-from six import (
-    reraise,
-    iteritems,
-    PY2,
-)
 from time import time
 
 from datalad import cfg
@@ -215,7 +210,7 @@ def _search_from_virgin_install(dataset, query):
                           % DEFAULT_DATASET_PATH):
                     pass
                 else:
-                    reraise(*exc_info)
+                    raise exc_info[1]
             else:
                 raise NoDatasetArgumentFound(
                     "No DataLad dataset found at current location. "
@@ -234,7 +229,7 @@ def _search_from_virgin_install(dataset, query):
                 "label '///'"
             )
         else:
-            reraise(*exc_info)
+            raise exc_info[1]
 
         lgr.info(
             "Performing search using DataLad superdataset %r",
@@ -281,7 +276,8 @@ class _WhooshSearch(_Search):
 
         self.idx_obj = None
         # where does the bunny have the eggs?
-        self.index_dir = opj(self.ds.path, GitRepo.get_git_dir(ds), SEARCH_INDEX_DOTGITDIR)
+
+        self.index_dir = opj(str(self.ds.repo.dot_git), SEARCH_INDEX_DOTGITDIR)
         self._mk_search_index(force_reindex)
 
     def show_keys(self, mode):
@@ -324,7 +320,7 @@ class _WhooshSearch(_Search):
         from .metadata import get_ds_aggregate_db_locations
         dbloc, db_base_path = get_ds_aggregate_db_locations(self.ds)
         # what is the lastest state of aggregated metadata
-        metadata_state = self.ds.repo.get_last_commit_hash(relpath(dbloc, start=self.ds.path))
+        metadata_state = self.ds.repo.get_last_commit_hexsha(relpath(dbloc, start=self.ds.path))
         # use location common to all index types, they would all invalidate
         # simultaneously
         stamp_fname = opj(self.index_dir, 'datalad_metadata_state')
@@ -726,7 +722,7 @@ class _EGrepCSSearch(_Search):
             t0 = time()
             matches = {(q['query'] if isinstance(q, dict) else q, k):
                        q['query'].search(v) if isinstance(q, dict) else q.search(v)
-                       for k, v in iteritems(doc)
+                       for k, v in doc.items()
                        for q in query
                        if not isinstance(q, dict) or q['field'].match(k)}
             dt = time() - t0
@@ -766,10 +762,7 @@ class _EGrepCSSearch(_Search):
 
         for k in sorted(keys):
             if mode == 'name':
-                from datalad.utils import assure_bytes
-                # without assure_bytes UnicodeEncodeError in PY2 when
-                # output is piped into e.g. grep
-                print(assure_bytes(k) if PY2 else k)
+                print(k)
                 continue
 
             # do a bit more
@@ -800,7 +793,7 @@ class _EGrepCSSearch(_Search):
                     "Unknown value for stats. Know full and short")
 
             print(
-                '{k}\n in  {stat.ndatasets} datasets\n has {stat.uvals_str}'.format(
+                u'{k}\n in  {stat.ndatasets} datasets\n has {stat.uvals_str}'.format(
                 k=k, stat=stat
             ))
         # After #2156 datasets may not necessarily carry all
@@ -836,7 +829,7 @@ class _EGrepCSSearch(_Search):
             # no stringification of values for speed
             idxd = _meta2autofield_dict(meta, val2str=False)
 
-            for k, kvals in iteritems(idxd):
+            for k, kvals in idxd.items():
                 # TODO deal with conflicting definitions when available
                 keys[k].ndatasets += 1
                 if mode == 'name':
